#version 400 core

const int MAX_BONES = 100;
const int MAX_BONE_INFLUENCE = 4;

layout(location = 0) in vec3 Position_VS_in;
layout(location = 1) in vec3 Normal_VS_in;
layout(location = 2) in vec2 Texture_VS_in;

layout(location = 3) in vec3 tangent;
layout(location = 4) in vec3 bitangent;
layout(location = 5) in ivec4 boneIds; 
layout(location = 6) in vec4 weights;

uniform mat4 projection;
uniform mat4 view;
uniform mat4 model;
uniform bool animated;

uniform mat4 finalBonesMatrices[MAX_BONES];

out vec2 Texture_CS_in;
out vec3 Position_CS_in;
out vec3 Normal_CS_in;

void main()
{
    if (animated)
    {      
        vec4 totalPosition = vec4(0.0f);
        for(int i = 0 ; i < MAX_BONE_INFLUENCE ; i++)
        {
            if(boneIds[i] == -1)
            {
                continue;
            }
            if(boneIds[i] >= MAX_BONES) 
            {
                totalPosition = vec4(Position_VS_in, 1.0f);
                break;
            }
            vec4 localPosition = finalBonesMatrices[boneIds[i]] * vec4(Position_VS_in,1.0f);
            totalPosition += localPosition * weights[i];
            vec3 localNormal = mat3(finalBonesMatrices[boneIds[i]]) * Normal_VS_in;
            }
        Position_CS_in = vec3(model * totalPosition);
        Normal_CS_in = mat3(model) * Normal_VS_in;
        gl_Position = projection * view * model * totalPosition;
        Texture_CS_in = Texture_VS_in;
    }
    else
    {
        Position_CS_in = mat3(model) * Position_VS_in;
        Normal_CS_in = mat3(model) * Normal_VS_in;
        gl_Position =  projection * view * model * vec4(Position_VS_in, 1.0f);
	    Texture_CS_in = Texture_VS_in;
    }
}